const Lang = imports.lang;
const St = imports.gi.St;
const Shell = imports.gi.Shell;
const Config = imports.misc.config;
const Main = imports.ui.main;
const Mainloop = imports.mainloop;
const MessageTray = Main.messageTray;
const LayoutManager = Main.layoutManager;
const Tweener = imports.ui.tweener;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

let panel = Main.layoutManager.panelBox;

const IDLE_TIME = 1000;
const ANIMATION_TIME = 0.2;
const Urgency = { LOW: 0, NORMAL: 1, HIGH: 2, CRITICAL: 3 };
const State = { HIDDEN: 0, SHOWING: 1, SHOWN: 2, HIDING: 3 };

let extensionShowNotification = function() {
    this._notification = this._notificationQueue.shift();

    this._userActiveWhileNotificationShown = this.idleMonitor.get_idletime() <= IDLE_TIME;
    if (ExtensionUtils.versionCheck(['3.6'], Config.PACKAGE_VERSION)) {
        this._idleMonitorWatchId = this.idleMonitor.add_watch(IDLE_TIME,
                                                              Lang.bind(this, this._onIdleMonitorWatch));
    }
    else
    {
        if (!this._userActiveWhileNotificationShown) {
            // If the user isn't active, set up a watch to let us know
            // when the user becomes active.
            this.idleMonitor.add_user_active_watch(Lang.bind(this, this._onIdleMonitorBecameActive));
        }
    }

    this._notificationClickedId = this._notification.connect('done-displaying',
                                                             Lang.bind(this, this._escapeTray));
    this._notificationUnfocusedId = this._notification.connect('unfocused', Lang.bind(this, function() {
        this._updateState();
    }));
    this._notificationBin.child = this._notification.actor;

    this._notificationWidget.opacity = 0;
    // JRL changes begin
    //this._notificationWidget.y = 0;
    let yTop = -global.screen_height;
    let yBottom = 0;

    this._notificationWidget.y = yTop;
    // JRL changes end


    this._notificationWidget.show();

    this._updateShowingNotification();

    let [x, y, mods] = global.get_pointer();
    // We save the position of the mouse at the time when we started showing the notification
    // in order to determine if the notification popped up under it. We make that check if
    // the user starts moving the mouse and _onTrayHoverChanged() gets called. We don't
    // expand the notification if it just happened to pop up under the mouse unless the user
    // explicitly mouses away from it and then mouses back in.
    this._showNotificationMouseX = x;
    this._showNotificationMouseY = y;
    // We save the coordinates of the mouse at the time when we started showing the notification
    // and then we update it in _notificationTimeout(). We don't pop down the notification if
    // the mouse is moving towards it or within it.
    this._lastSeenMouseX = x;
    this._lastSeenMouseY = y;
};

let extensionHideNotification = function(animate) {
    if (ExtensionUtils.versionCheck(['3.6', '3.8'], Config.PACKAGE_VERSION)) {
        // HACK!
        // There seems to be a reentrancy issue in calling .ungrab() here,
        // which causes _updateState to be called before _notificationState
        // becomes HIDING. That hides the notification again, nullifying the
        // object but not setting _notificationState (and that's the weird part)
        // As then _notificationState is stuck into SHOWN but _notification
        // is null, every new _updateState fails and the message tray is
        // lost forever.
        //
        // See more at https://bugzilla.gnome.org/show_bug.cgi?id=683986
        this._notificationState = State.HIDING;

        this._grabHelper.ungrab({ actor: this._notification.actor });
    }
    else
    {
        this._notificationFocusGrabber.ungrabFocus();
    }

    if (this._notificationExpandedId) {
        this._notification.disconnect(this._notificationExpandedId);
        this._notificationExpandedId = 0;
    }
    // JRL changes begin
    let yPos = -global.screen_height;
    // JRL changes end
    if (ExtensionUtils.versionCheck(['3.6'], Config.PACKAGE_VERSION)) {

        if (this._notificationRemoved) {
            // JRL changes begin
            //this._notificationWidget.y = this.actor.height;
            this._notificationWidget.y = yPos;
            // JRL changes end
            this._notificationWidget.opacity = 0;
            this._notificationState = State.HIDDEN;
            this._hideNotificationCompleted();
        } else {
            this._tween(this._notificationWidget, '_notificationState', State.HIDDEN,
                        // JRL changes begin
                        //{ y: this.actor.height,
                        { y: yPos,
                        // JRL changes end
                          opacity: 0,
                          time: ANIMATION_TIME,
                          transition: 'easeOutQuad',
                          onComplete: this._hideNotificationCompleted,
                          onCompleteScope: this
                        });

        }
    }
    else
    {
        if (this._notificationClickedId) {
            this._notification.disconnect(this._notificationClickedId);
            this._notificationClickedId = 0;
        }
        if (this._notificationUnfocusedId) {
            this._notification.disconnect(this._notificationUnfocusedId);
            this._notificationUnfocusedId = 0;
        }

        if (ExtensionUtils.versionCheck(['3.8'], Config.PACKAGE_VERSION)) {
            this._useLongerTrayLeftTimeout = false;
            if (this._trayLeftTimeoutId) {
                Mainloop.source_remove(this._trayLeftTimeoutId);
                this._trayLeftTimeoutId = 0;
                this._trayLeftMouseX = -1;
                this._trayLeftMouseY = -1;
            }

            if (this._notificationRemoved) {
                Tweener.removeTweens(this._notificationWidget);
                // JRL changes begin
                //this._notificationWidget.y = this.actor.height;
                this._notificationWidget.y = yPos;
                // JRL changes end
                this._notificationWidget.opacity = 0;
                this._notificationState = State.HIDDEN;
                this._hideNotificationCompleted();
            } else {
                this._tween(this._notificationWidget, '_notificationState', State.HIDDEN,
                            // JRL changes begin
                            //{ y: this.actor.height,
                            { y: yPos,
                            // JRL changes end
                              opacity: 0,
                              time: ANIMATION_TIME,
                              transition: 'easeOutQuad',
                              onComplete: this._hideNotificationCompleted,
                              onCompleteScope: this
                            });

            }
        }
        else
        {
            if (this._notificationLeftTimeoutId) {
                Mainloop.source_remove(this._notificationLeftTimeoutId);
                this._notificationLeftTimeoutId = 0;
                this._notificationLeftMouseX = -1;
                this._notificationLeftMouseY = -1;
            }

            if (animate) {
                this._tween(this._notificationWidget, '_notificationState', State.HIDDEN,
                            // JRL changes begin
                            //{ y: this.actor.height,
                            { y: yPos,
                            // JRL changes end
                              opacity: 0,
                              time: ANIMATION_TIME,
                              transition: 'easeOutQuad',
                              onComplete: this._hideNotificationCompleted,
                              onCompleteScope: this
                            });
            } else {
                Tweener.removeTweens(this._notificationWidget);
                // JRL changes begin
                //this._notificationWidget.y = this.actor.height;
                this._notificationWidget.y = yPos;
                // JRL changes end
                this._notificationWidget.opacity = 0;
                this._notificationState = State.HIDDEN;
                this._hideNotificationCompleted();
            }
        }
    }
};

let extensionUpdateShowingNotification = function() {
    // JRL changes begin
    // add own class-name to change border-radius, otherwise the changed value remains after switching off the extension
    this._notification._table.remove_style_class_name('jrlnotification');
    this._notification._table.remove_style_class_name('jrlnotification_top');
    this._notification._table.remove_style_class_name('jrlnotification_bottom');
    this._notification._table.add_style_class_name('jrlnotification_top');

    // JRL changes end
    this._notification.acknowledged = true;
    if (ExtensionUtils.versionCheck(['3.6'], Config.PACKAGE_VERSION)) {
        // We auto-expand notifications with CRITICAL urgency.
        // JRL changes begin
        //        if (this._notification.urgency == Urgency.CRITICAL)
        if (this._notification.urgency == Urgency.CRITICAL)
        // JRL changes end
            this._expandNotification(true);
    }
    else
    {
        this._notification.playSound();
        // We auto-expand notifications with CRITICAL urgency, or for which the relevant setting
        // is on in the control center.
        if (this._notification.urgency == Urgency.CRITICAL ||
            this._notification.source.policy.forceExpanded)
            this._expandNotification(true);
    }

    // JRL changes begin
    // use panel's y and height property to determine the bottom of the top-panel.
    // needed because the "hide top bar" and "hide top panel" use different approaches to hide the
    // top bar.
    // "hide top panel" keeps the height and just moves the panel out of the visible area, so using
    // the panels-height is not enough.
    let yTop = panel.y + panel.height - global.screen_height;
    if (yTop < (-global.screen_height))
        yTop = -global.screen_height;
    let yBottom = -this._notificationWidget.height;

    this._notificationWidget.x = global.screen_width / 50;
    // JRL changes end
    // We tween all notifications to full opacity. This ensures that both new notifications and
    // notifications that might have been in the process of hiding get full opacity.
    //
    // We tween any notification showing in the banner mode to the appropriate height
    // (which is banner height or expanded height, depending on the notification state)
    // This ensures that both new notifications and notifications in the banner mode that might
    // have been in the process of hiding are shown with the correct height.
    //
    // We use this._showNotificationCompleted() onComplete callback to extend the time the updated
    // notification is being shown.

    let tweenParams = { opacity: 255,
                        // JRL changes begin
                        //y: -this._notificationWidget.height,
                        y: yTop,
                        // JRL changes end
                        time: ANIMATION_TIME,
                        transition: 'easeOutQuad',
                        onComplete: this._showNotificationCompleted,
                        onCompleteScope: this
                      };

    this._tween(this._notificationWidget, '_notificationState', State.SHOWN, tweenParams);
};

let extensiononNotificationExpanded = function() {
    // JRL changes begin
    //let expandedY = - this._notificationWidget.height;
    let yTop = panel.y + panel.height - global.screen_height;
    if (yTop < (-global.screen_height))
        yTop = -global.screen_height;
    let yBottom = -this._notificationWidget.height;

    let expandedY = yTop;
    // JRL changes end
    this._closeButton.show();

    // Don't animate the notification to its new position if it has shrunk:
    // there will be a very visible "gap" that breaks the illusion.
    if (this._notificationWidget.y < expandedY) {
        this._notificationWidget.y = expandedY;
    } else if (this._notification.y != expandedY) {
        // Tween also opacity here, to override a possible tween that's
        // currently hiding the notification.
        this._tween(this._notificationWidget, '_notificationState', State.SHOWN,
                    { y: expandedY,
                      opacity: 255,
                      time: ANIMATION_TIME,
                      transition: 'easeOutQuad'
                    });
    }
};

const ChangeNT = new Lang.Class({
    Name: 'ChangeNT',

    _init: function(enabled) {

		this.showTestNotificationTimeout = undefined;

		this.originalNotificationWidgetX = Main.messageTray._notificationWidget.x;
		this.originalExpandMethod = MessageTray._onNotificationExpanded;
		this.originalShowNotification = MessageTray._showNotification;
		this.originalUpdateShowingNotification = MessageTray._updateShowingNotification;
		this.originalHideNotification = MessageTray._hideNotification;

        if (enabled) {
            this.enable(false);
        }
    },

    disable: function() {

    	if (this.showTestNotificationTimeout !== undefined)
        	Mainloop.source_remove(this.showTestNotificationTimeout);

		if (MessageTray._notification) {
		    MessageTray._notification._table.remove_style_class_name('jrlnotification');
		    MessageTray._notification._table.remove_style_class_name('jrlnotification_top');
		    MessageTray._notification._table.remove_style_class_name('jrlnotification_bottom');
		}

		Main.messageTray._notificationWidget.x = this.originalNotificationWidgetX;
		MessageTray._showNotification = this.originalShowNotification;
		MessageTray._hideNotification = this.originalHideNotification;
		MessageTray._updateShowingNotification = this.originalUpdateShowingNotification;
		MessageTray._onNotificationExpanded = this.originalExpandMethod;
    },

    enable: function(test) {

		MessageTray._showNotification = extensionShowNotification;
		MessageTray._hideNotification = extensionHideNotification;
		MessageTray._updateShowingNotification = extensionUpdateShowingNotification;
		MessageTray._onNotificationExpanded = extensiononNotificationExpanded;

		if (test) {
			this.test();
		}
    },

	test: function() {
        if (this.showTestNotificationTimeout !== undefined)
            Mainloop.source_remove(this.showTestNotificationTimeout);

    	this.showTestNotificationTimeout = Mainloop.timeout_add(250,
			Lang.bind(this, function() {
				Main.notify("Simple Dock", "This is just a notification test.");
        		return false;
    		}));
	},

    destroy: function() {
		this.disable();
    }
});

